@@ -1,7 +1,7 @@
 #######################################################################
-# $Date: 2007-07-20 15:04:55 -0700 (Fri, 20 Jul 2007) $
-# $Revision: 168 $
-# $Author: david.romano $
+# $Date$
+# $Revision$
+# $Author$
 # ex: set ts=8 sw=4 et
 #########################################################################
 package WWW::Facebook::API::Auth;
@@ -10,7 +10,7 @@ use warnings;
 use strict;
 use Carp;
 
-use version; our $VERSION = qv('0.4.4');
+use version; our $VERSION = qv('0.4.5');
 
 sub create_token {
     my $self = shift;
@@ -34,39 +34,34 @@ sub get_session {
 
     my $token = shift;
     croak q{Token needed for call to get_session} if not defined $token;
-    if ( $self->base->desktop ) {
-        ( my $uri_https = $self->base->server_uri )
-            =~ s{http://}{https://}xms;
-        $self->base->server_uri($uri_https);
-    }
-
-    my ( $format, $parse ) = ( $self->base->format, $self->base->parse );
-
-    $self->base->format('JSON');
-    $self->base->parse(0);
-
-    my $response =
-        $self->base->call( 'auth.getSession', auth_token => $token );
-
-    $self->base->format($format);
-    $self->base->parse($parse);
 
+    # Mappings between the response fields and the WFA fields
     my %field = qw(
         session_key     session_key
         expires         session_expires
         uid             session_uid
+        secret          secret
     );
 
-    if ( $self->base->desktop ) {
-        $field{'secret'} = 'secret';
-        ( my $uri_http = $self->base->server_uri ) =~ s{https://}{http://}xms;
-        $self->base->server_uri($uri_http);
-    }
+    # Desktop apps need to call the https rather than http endpoint
+    $self->_use_https_uri() if $self->base->desktop;
 
-    while ( my ( $key, $val ) = each %field ) {
-        $response =~ /$key"\W+([\w-]+)/xms;
-        carp "Setting $key to $1" if $self->base->debug;
-        $self->base->$val($1);    ## no critic
+    # Save format and parse settings, and get parsed response from server
+    my ( $format, $parse ) = ( $self->base->format, $self->base->parse );
+    $self->base->format('JSON');
+    $self->base->parse(1);
+    my $resp = $self->base->call( 'auth.getSession', auth_token => $token );
+    $self->base->format($format);
+    $self->base->parse($parse);
+
+    $self->_use_http_uri() if $self->base->desktop;
+
+    # Copy values from response to object's hash
+    AUTH_RESPONSE:
+    for my $key ( keys %{$resp} ) {
+        next AUTH_RESPONSE if $key eq 'secret' && !$self->base->desktop;
+        carp "Setting $field{$key}: $resp->{$key}" if $self->base->debug;
+        $self->base->{ $field{$key} } = $resp->{$key};
     }
 
     return;
@@ -105,6 +100,20 @@ sub logout {
     return;
 }
 
+sub _use_http_uri  { return shift->_flip_scheme(0); } ## no critic
+sub _use_https_uri { return shift->_flip_scheme(1); } ## no critic
+
+sub _flip_scheme {
+    my $self = shift;
+    my $make_https = shift;
+    my $scheme = $make_https ? 'http' : 'https';
+
+    ( my $uri = $self->base->server_uri() ) =~ s{^[^:]+:}{$scheme:}xms;
+    $self->base->server_uri($uri);
+
+    return;
+}
+
 1;
 __END__
 
@@ -171,6 +180,26 @@ http://developers.facebook.com/documentation.php?v=1.0&doc=auth )
 
 =back
 
+=head1 PRIVATE METHODS
+
+=over
+
+=item _use_http_uri()
+
+Makes the WFA object's server URI scheme http. Uses C<_flip_scheme>.
+
+=item _use_https_uri()
+
+Makes the WFA object's server URI scheme https. Uses C<_flip_scheme>.
+
+=item _flip_scheme( $make_https )
+
+If C<$make_https> is true, the scheme becomes https. If false, the scheme
+becomes http. The WFA object's C<server_uri> attribute is then set to use that
+scheme.
+
+=back
+
 =head1 DIAGNOSTICS
 
 =over
